#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "color_correct.h"
#include <libgimp/gimp.h>
#include <libgimp/gimpui.h>
#include "data.h"
#include "colorconv.h"

static void on_greyworld_clicked (GtkButton * button, gpointer user_data);

static void on_FGBG_clicked (GtkButton * button, gpointer user_data);

static void on_autostretch_clicked (GtkButton * button, gpointer user_data);

extern GimpDrawable *src_drawable_global;

GtkWidget *
automagic_ui (void)
{
  GtkTooltips *tooltips;
  GtkWidget *vbox;
  GtkWidget *frame;
  GtkWidget *ivbox;
  GtkWidget *button;
  GtkWidget *text;

  tooltips = gtk_tooltips_new ();

  vbox = gtk_vbox_new (FALSE, 1);
  gtk_container_set_border_width (GTK_CONTAINER (vbox), 1);

  frame = gtk_frame_new (_("Automatic Whitebalance"));
  gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, 0);

  ivbox = gtk_vbox_new (FALSE, 1);
  gtk_container_add (GTK_CONTAINER (frame), ivbox);

  text = gtk_label_new (_("Assume that the average color of the image is some kind of gray, and set the values in the color-balance tab accordincly"));
  gtk_label_set_line_wrap (GTK_LABEL (text), TRUE);
  gtk_label_set_justify (GTK_LABEL (text), GTK_JUSTIFY_LEFT);

  gtk_box_pack_start (GTK_BOX (ivbox), text, TRUE, TRUE, 0);

  button = gtk_button_new_with_label (_("Apply"));

  gtk_box_pack_end (GTK_BOX (ivbox), button, FALSE, FALSE, 0);

  g_signal_connect (GTK_OBJECT (button), "clicked",
                    GTK_SIGNAL_FUNC (on_greyworld_clicked), NULL);


  frame = gtk_frame_new (_("Match white/black"));
  gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, 0);

  ivbox = gtk_vbox_new (FALSE, 1);
  gtk_container_add (GTK_CONTAINER (frame), ivbox);

  text = gtk_label_new (_
      ("Use the color picker tool to get a color known to be white in the image, and set it as the foreground color.\n"
       "a known black color (or pure black) should be used as the background color when pressing apply, the plug-in will "
       "adjust itself to reflect the values that would counter the colorcast in the image.")
  );
  gtk_label_set_line_wrap (GTK_LABEL (text), TRUE);
  gtk_label_set_justify (GTK_LABEL (text), GTK_JUSTIFY_LEFT);

  gtk_box_pack_start (GTK_BOX (ivbox), text, TRUE, TRUE, 0);

  button = gtk_button_new_with_label (_("Apply"));

  gtk_box_pack_end (GTK_BOX (ivbox), button, FALSE, FALSE, 0);

  g_signal_connect (GTK_OBJECT (button), "clicked",
                    GTK_SIGNAL_FUNC (on_FGBG_clicked), NULL);


  frame = gtk_frame_new (_("Stretch contrast"));
  gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, 0);

  ivbox = gtk_vbox_new (FALSE, 1);
  gtk_container_add (GTK_CONTAINER (frame), ivbox);

  text = gtk_label_new (
       _("Set the start and end of the levels input to be the brightest, and the darkest values found in the image")
  );
  gtk_label_set_line_wrap (GTK_LABEL (text), TRUE);
  gtk_label_set_justify (GTK_LABEL (text), GTK_JUSTIFY_LEFT);

  gtk_box_pack_start (GTK_BOX (ivbox), text, TRUE, TRUE, 0);

  button = gtk_button_new_with_label (_("Apply"));

  gtk_box_pack_end (GTK_BOX (ivbox), button, FALSE, FALSE, 0);

  g_signal_connect (GTK_OBJECT (button), "clicked",
            GTK_SIGNAL_FUNC (on_autostretch_clicked), NULL);



  return vbox;
}

static void
on_greyworld_clicked (GtkButton *button,
                      gpointer   user_data)
{
  calculate_greyworld ();
  update_values ();
}


static void
on_autostretch_clicked (GtkButton *button,
                        gpointer   user_data)
{
  GimpPixelRgn rgn;
  gpointer pr;
  gint brightest = 0, darkest = 255;

  gimp_pixel_rgn_init (&rgn, src_drawable_global, 0, 0,
             src_drawable_global->width,
             src_drawable_global->height, FALSE, FALSE);

  for (pr = gimp_pixel_rgns_register (1, &rgn);
       pr != NULL; pr = gimp_pixel_rgns_process (pr))
    {
      int x, y;
      guchar *src = rgn.data;

      for (y = 0; y < rgn.h; y++)
        {
          guchar *s = src;

          for (x = 0; x < rgn.w; x++)
            {
              int R, G, B, Y, U, V;

              R = s[0];
              G = s[1];
              B = s[2];
              RGB2YUV (R, G, B, Y, U, V);
              if (Y > brightest)
                brightest = Y;
              if (Y < darkest)
                darkest = Y;

              s += rgn.bpp;
            }
          src += rgn.rowstride;
        }
    }

  pv.by = darkest;
  pv.wy = brightest;

  pv.gy = (darkest + brightest) / 2;

  update_values ();
}

static void
on_FGBG_clicked (GtkButton *button,
                 gpointer   user_data)
{
  int R, G, B;
  int Y, U, V;

  GimpRGB fgcol;
  GimpRGB bgcol;

  gimp_palette_get_foreground (&fgcol);
  gimp_palette_get_background (&bgcol);

  R = fgcol.r * 255;
  G = fgcol.g * 255;
  B = fgcol.b * 255;

  RGB2YUV (R, G, B, Y, U, V);

  pv.wu = -(U - 128);
  pv.wv = -(V - 128);
  pv.wy = Y;

  R = bgcol.r * 255;
  G = bgcol.g * 255;
  B = bgcol.b * 255;

  RGB2YUV (R, G, B, Y, U, V);

  pv.bu = -(U - 128);
  pv.bv = -(V - 128);
  pv.by = Y;

  pv.gy = (pv.by + pv.wy) / 2;
  pv.gu = (pv.bu + pv.wu) / 2;
  pv.gv = (pv.bv + pv.wv) / 2;

  update_values ();
}
